%% 
% Polynomial Basis Pursuit based on Selective l1/l2-minimization
%
% by F. Lauer, 2013. 
%
%
%  [x, phi, yhat, sparsity, x1] = polybpSL1L2M(Q, y, n, deg, verbose)
%
%	Compute x = phi(1:n)
%
%	with 	phi = argmin  sum_j  mu_j || W_j Phi ||_2
%
%					s.t. y = A Phi + b 
%
%	and  	mu_j in {0,1} iteratively updated by Selective L1 minimization. 
%
% Inputs (N = nb of equations; M = dim(phi) ):
%	
%	Q 		: N-by-(M+1) matrix of polynomial coefficients 
%				Q(:,1) = b ;  Q(:,2:end) = A
%   y 		: N-by-1 vector of measurements
%   n		: number of variables = dim(x)
%   deg 	: maximal degree of the polynomials 
%	verbose : print info at each iteration if true
%
% Outputs:
%	x   	: n-vector estimate of x
%	phi 	: M-vector solution to the optimization problem (estimates of the monomials)
%	yhat	: N-vector approximation of y as yhat = A phi + b
%	sparsity: n-vector with support corresponding to the nonzero norms || W_j phi ||
%
%	x1: solution obtained at the first iteration (no reweighting)
%
% The columns in Q must be ordered such that Q(:,k) contains the 
% coefficients associated to the monomial x^(alpha_k) = prod(x.^(alpha(k,:)'))
% with alpha = multiindex(n, deg). 
%
function [xhat, phihat, yhat, t, xhat1] = polybpSL1L2M(Q,y,n,degree, verbose)

if nargin < 5
	verbose = false; 
end

N = size(Q,1);

alpha = multiindex(n, degree);
dim_alpha = size(alpha,1);
dim = dim_alpha - 1;

% conic constraints
dim_uj = sum(alpha(:,1)~=0); 

tj = dim + dim_uj*n;
start_u = dim;

nvars = dim + dim_uj*n + n;

prob.cones = cell(n,1);
	
% nb constraints uj_l = phi_k:
ncst = sum(sum(alpha~=0));
	
Aeq = [Q(:,2:end) spalloc(N,nvars - dim,0)];
Beq = [y - Q(:,1) ; zeros(dim_uj*n,1)];
row = N + 1;
	
nnzeros = 2*ncst;
sparse_row = zeros(nnzeros,1); 
sparse_col = zeros(nnzeros,1); 
sparse_value = zeros(nnzeros,1); 
sparse_n = 1; 

for j=1:n	
	l = 1;

	idx = find(alpha(:,j)~=0);
	
	for k=1:length(idx)  
	
			% add constraint uj_l = phi_k 
			
			sparse_row(sparse_n) = row-N;
			sparse_col(sparse_n) = (start_u + (j-1)*dim_uj + l) ;
			sparse_value(sparse_n) = 1;
			sparse_n = sparse_n + 1;
			
			sparse_row(sparse_n) = row-N;
			sparse_col(sparse_n) = idx(k)-1;
			sparse_value(sparse_n) = -1;
			sparse_n = sparse_n + 1;

			row = row + 1;

			l= l + 1;
	end
	
	tj = tj + 1;
	prob.cones{j}.type = 'MSK_CT_QUAD';	
	prob.cones{j}.sub  = [tj, start_u + (j-1)*dim_uj+1:start_u + j*dim_uj];
end

Aeqcst = sparse(sparse_row,sparse_col,sparse_value,ncst,nvars);

% Optimize with MOSEK

% constraints
prob.a  = sparse([Aeq;Aeqcst]);
clear Aeq;
prob.blc = [Beq'];
prob.buc = [Beq'];

% Bounds on variables
UB = inf*ones(nvars,1);
LB = [-inf*ones(nvars-n,1); zeros(n,1)];

% positivity constraints on "even" monomials 
positive = find(sum(mod(alpha(2:end,:),2),2) == 0) ; 
LB(positive) = 0;

prob.blx = LB';
prob.bux = UB';

% Solve SOCP

W = ones(n,1);
phihat = zeros(dim,1);
xhat = ones(n,1);
t = ones(n,1);
iter = 0;
while sum(W) > 0 & sum(abs(t(W~=0))) > 1e-6
	iter = iter + 1;

	% reweighted cost 
	lincost = [zeros(dim + dim_uj*n,1); W];
	
	prob.c = sparse(lincost);

	[r, res] = mosekopt('minimize echo(0)',prob);
	sol = res.sol.itr.xx;

	if sum(sol(1:dim) == phihat)==dim , break, end
	
	phihat = sol(1:dim);

	xhat = phihat(1:n);

	if iter == 1
		xhat1 = xhat;
	end

	yhat = Q * [1;phihat];

	t = sol(nvars -n+1: end);

	[tmax, idx] = max(t.*W);

	W(idx) = 0;

%  perhaps speed up by removing useless cones and constraints... 

	if verbose
		xhat
		sparsityFactorNorms = sum(abs(t) < 1e-6) / n
		sparsityFactorX = sum(abs(xhat) < 1e-6) / n
	
		SumSquaredError = checksol(xhat,Q,y,n,degree)
	end
end
