%% 
% Greedy algorithm for finding sparse solutions of a polynomial system 
%
% by F. Lauer, 2013. 
%
%  [x, S, err, phi] = polygreedyexact(Q, y, n, deg, epsilon)
%
%	Compute x = phi(1:n)
%
%	with 	phi an EXACT solution to
%
%				min  sum_j || { W_j Phi }_j=1^n ||_0
%
%					s.t. || A Phi + b - y ||_2 < epsilon 
%
% Inputs (N = nb of equations; M = dim(phi) ):
%	
%	Q 		: N-by-(M+1) matrix of polynomial coefficients 
%				Q(:,1) = b ;  Q(:,2:end) = A
%   y 		: N-by-1 vector of measurements
%   n		: number of variables = dim(x)
%   deg 	: maximal degree of the polynomials 
%   epsilon	: bound on the l2-norm of the error 
%	verbose : print info at each iteration if true
%
% Outputs:
%	x   	: n-vector estimate of x
%	S		: support of x 
%	err		: remaining error || A phi + b - y ||_2^2
%	phi 	: M-vector with estimates of the monomials
%
% The columns in Q must be ordered such that Q(:,k) contains the 
% coefficients associated to the monomial x^(alpha_k) = prod(x.^(alpha(k,:)'))
% with alpha = multiindex(n, deg). 
%
function [xhat, vars, err, phihat] = polygreedyexact(Q,y,n,degree, epsilon)
	warning off;
	
	if nargin < 5
		epsilon = 1e-3;   % tol on remaining error for A*phi -y		
	end 
	
	epsilon2 = epsilon .* epsilon;
	
	N = size(Q,1);

	alpha = multiindex(n, degree);
	dim_alpha = size(alpha,1);
	dim = dim_alpha - 1;
	
	
	yb = y - Q(:,1);
	
	phihat = zeros(dim_alpha,1);
	
	err = sum(yb.^2);
	
	
	% FOR ALL number of variables		
	nbvars = 0;
	vars = [];
	while nbvars <= n  & err > epsilon2

		nbvars = nbvars + 1;

		combinations = nchoosek(1:n, nbvars); 
	
		e = zeros(1,size(combinations,1));
		
		% FOR all combinations with this nb of vars
		for k=1:size(combinations,1) 

			S = support(combinations(k,:), alpha);
			A = Q(:,S) ;
			phi = A \ yb;
			
			e(k) = sum(( A * phi - yb).^2);
			
			if e(k) <= epsilon2 
				phihat( S ) = phi;
				vars = combinations(k,:);
			end
		end
		err = min(e);
		
	end
	

	phihat(1) = [];
	
	xhat = phihat(1:n);
	
end

function supp = support(idx, alpha) 
	notidx = (1:size(alpha,2));
	notidx(idx) = [];
	supp = find(sum(alpha(:,notidx),2) == 0); 
	supp(1) = [];
	
end

